#!/usr/bin/env python
import os
import sys
import ast
import subprocess
import traceback
import time

from charmhelpers.core import unitdata
from charmhelpers.core.host import set_nic_mtu
from charmhelpers.core.hookenv import (
    log, ERROR, config, relation_ids, relation_get,
    relation_set, Hooks, status_set, related_units,
    UnregisteredHookError, service_name, unit_private_ip,
    local_unit)

from charmhelpers.contrib.openstack.ip import (canonical_url,
                                               PUBLIC,
                                               INTERNAL,
                                               ADMIN)

from charmhelpers.contrib.python.packages import (pip_install)

dest_dir = os.path.join(os.environ.get('CHARM_DIR'), 'libs')
wheel_file = os.path.join(dest_dir, 'nexentaedge-1.0-py2-none-any.whl')
pip_install(wheel_file)

from nexentaedge.nedgeConfigurator import (NedgeGatewayConfigurator)
from nexentaedge.settings import Settings
from nexentaedge.utils import (get_sid)

from utils import (neadm_bundles_copy)


hooks = Hooks()
hook_data = unitdata.HookData()
db = unitdata.kv()


@hooks.hook('install')
def install():
    try:
        neadm_bundles_copy()
        log('Bundles has copied')
    except subprocess.CalledProcessError as cpe:
        log('Install failed!\nMessage:\n{0}\nTrace:\n{1}\nOutput:\n{2}'
            .format(cpe.message, traceback.format_exc(), cpe.output))
        sys.exit(1)
    except Exception as e:
        log('Installation failed!\nMessage:{0}\nTrace:{1}'
            .format(e.message, traceback.format_exc()))
        sys.exit(1)


@hooks.hook('config-changed')
def on_config_changed():
    if get_sid():
        log('Nexentaedge node already installed...')
        return

    private_ip = unit_private_ip()
    replicast_eth = config('replicast_eth')

    log('Node private ip is {}'.format(private_ip))
    environment = {
        'node_private_ip': private_ip,
        'replicast_eth': replicast_eth,
        'profile': config('profile'),
        'nodocker': config('nodocker'),
        'exclude': config('exclude'),
        'reserved': config('reserved')
    }

    #build 2240, should set replicast MTU to 9000
    replicast_eth = config('replicast_eth')
    if replicast_eth:
        set_nic_mtu(replicast_eth, '9000')

    status_set('maintenance', 'Installing nexentaedge...')
    configurator = NedgeGatewayConfigurator(environment)
    if not configurator.configure():
        blockers = configurator.get_blockers()
        if blockers:
            status_set('blocked', blockers[0])
            sys.exit(0)
        log('NEDGE swift gateway node configuration failed.', level=ERROR)
        sys.exit(1)

    time.sleep(60)
    #resend data to mgmt if relation already created
    for relid in relation_ids('swift-gw'):
        for unit in related_units(relid):
            log('resend data for relid:{0} unit:{1}'.format(relid, unit))
            on_mgmt_joined(relid=relid, unit=unit)
    status_set('active', 'Ready')


@hooks.hook('swift-gw-relation-joined')
def on_mgmt_joined(relid=None, unit=None):

    if not get_sid():
        return

    # send node sid to mgmt to create new service
    data = {'gw_info': {'sid': get_sid(),
                        'unit_id': local_unit(),
                        'service_name': service_name()}}
    log("send gw_info:{} to MGMT".format(data))
    relation_set(relation_id=relid,
                 relation_settings=data)


@hooks.hook('swift-gw-relation-departed')
def on_mgmt_departed():
    # mark service as detached
    context = db.get('swift-context', record=True)
    log("context data is {}".format(context))
    if context:
        context['state'] = 'detached'
        db.set('swift-context', context)


@hooks.hook('swift-gw-relation-changed')
def on_swift_gateway_data_changed():
    # recive and parse message from mgmt
    context = relation_get('swift-context')
    log("data recived from MGMT: {}".format(context))
    if context:
        context = ast.literal_eval(context)
        if context['sid'] == get_sid() and context['state'] == 'incomplete':

            tenant = context['tenant']
            port = Settings.KEYSTONE_PORT
            context['operator-roles'] = config('operator-roles')
            context['region'] = config('region')
            context['public_url'] = "{0}:{1}/v1/{2}".format(
                canonical_url(None, PUBLIC), port, tenant)
            context['internal_url'] = "{0}:{1}/v1/{2}".format(
                canonical_url(None, INTERNAL), port, tenant)
            context['admin_url'] = "{0}:{1}".format(
                canonical_url(None, ADMIN), port)
            context['state'] = 'complete'

            db.set('swift-context', context)
            log("Swift context is:\n{}".format(context))

            for relid in relation_ids('identity-service'):
                identity_service_joined(relid)
        else:
            log("WRONG data: {}".format(context))


@hooks.hook('identity-service-relation-joined')
def identity_service_joined(relid=None):
    log("keysone identity service joined")
    context = db.get('swift-context', record=True)
    if context and context['state'] == 'complete':
        # send swift info to keystone

        log("send swift-context:{} to keystone".format(context))
        public_url = context['public_url']
        internal_url = context['internal_url']
        admin_url = context['admin_url']
        operator_roles = context['operator-roles']
        region = context['region']

        log("Swift context is:")
        log("public_url:{}".format(public_url))
        log("internal_url:{}".format(internal_url))
        log("admin_url:{}".format(admin_url))
        log("operator_roles:{}".format(operator_roles))
        log("region:{}".format(region))

        # send data to keystone
        relation_set(service='swift',
                     region=region,
                     public_url=public_url,
                     internal_url=internal_url,
                     admin_url=admin_url,
                     requested_roles=operator_roles,
                     relation_id=relid)

    else:
        log("[GW] No compete cinder-context to send to cinder-nedge")


if __name__ == '__main__':
    try:
        with hook_data():
            hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log('Unknown hook {} - skipping.'.format(e))
